using System;
using System.IO;

namespace AVIInfo
{
	/// <summary>
	/// Summary description for BSIWrapper.
	/// </summary>
	public class BSIWrapper
	{
        private string m_sAVIFile;
        private string m_sDirectory;
        private string m_sBSIFile;
        private string m_sNewChapters;

		public BSIWrapper()
		{
		}

        public void SetDefaults()
        {
            m_sAVIFile = "";
            m_sDirectory = "";
            m_sBSIFile = "";
            m_sNewChapters = "";
        }

        public bool SetFile(string sFile)
        {
            SetDefaults();

            m_sBSIFile = sFile;

            System.IO.TextReader reader = null;

            try
            {
                reader = new StreamReader(sFile, System.Text.Encoding.Default);
            }
            catch
            {
                return false;
            }

            ParseBSI(reader);

            reader.Close();

            return true;
        }

        private void ParseBSI(TextReader reader)
        {
            BSIState state = BSIState.Initial;

            while (reader.Peek() != -1)
            {
                string sLine = reader.ReadLine();

                state = CheckState(sLine, state);

                switch(state)
                {
                    case BSIState.Initial:
                        break;
                    case BSIState.Main:
                        break;
                    case BSIState.Movie:
                        ParseMovie(reader);
                        break;
                    case BSIState.Subtitles:
                        break;
                    case BSIState.Chapters:
                        break;
                }
            }
        }

        private BSIState CheckState(string sLine, BSIState state)
        {
            string sLower = sLine.ToLower();

            if (sLower == "[main]")
                return BSIState.Main;

            if (sLower == "[movie]")
                return BSIState.Movie;

            if (sLower == "[subtitles]")
                return BSIState.Subtitles;

            if (sLower == "[chapters]")
                return BSIState.Chapters;

            return state;
        }

        private void ParseMovie(TextReader reader)
        {
            while (reader.Peek() != '[')
            {
                if (reader.Peek() == -1)
                    break;

                string sLine = reader.ReadLine();
                string sLower = sLine.ToLower();

                if (sLower.StartsWith("directory="))
                {
                    m_sDirectory = sLine.Substring(sLine.IndexOf('=') + 1);
                }
                else if (sLower.StartsWith("fname="))
                {
                    m_sAVIFile = sLine.Substring(sLine.IndexOf('=') + 1);
                }
            }
        }

        public bool RoundChapterFrames(long[] Keyframes)
        {
            BSIState state = BSIState.Initial;
            TextReader reader = null;

            try
            {
                reader = new StreamReader(m_sBSIFile, System.Text.Encoding.Default);
            }
            catch
            {
                return false;
            }

            while (reader.Peek() != -1)
            {
                string sLine = reader.ReadLine();

                state = CheckState(sLine, state);

                switch(state)
                {
                    case BSIState.Initial:
                        break;
                    case BSIState.Main:
                        break;
                    case BSIState.Movie:
                        break;
                    case BSIState.Subtitles:
                        break;
                    case BSIState.Chapters:
                        m_sNewChapters = FixChapters(reader, Keyframes);
                        break;
                }
            }

            reader.Close();

            return true;
        }

        private string FixChapters(TextReader reader, long[] Keyframes)
        {
            string sNew = "[Chapters]\r\n";

            while (reader.Peek() != '[')
            {
                if (reader.Peek() == -1)
                    break;

                string sLine = reader.ReadLine();
                int iEquals = sLine.IndexOf('=');
                if (iEquals > -1)
                {
                    int iSpace = sLine.IndexOf(' ', iEquals);
                    if (iSpace > -1)
                    {
                        string sPrefix = sLine.Substring(0, iEquals + 1);
                        string sSuffix = sLine.Substring(iSpace);
                        long iFrame = Int32.Parse(sLine.Substring(iEquals + 1, iSpace - iEquals - 1));

                        iFrame = NearestKeyframe(iFrame, Keyframes);

                        sNew += string.Format("{0}{1}{2}\r\n", sPrefix, iFrame, sSuffix);
                    }
                }
                else
                {
                    sNew += (sLine + "\r\n");
                }
            }

            return sNew;
        }

        private long NearestKeyframe(long iFrame, long[] Keyframes)
        {
            long iClosest = 0;

            foreach(long lKey in Keyframes)
            {
                if (Math.Abs(iFrame - lKey) < Math.Abs(iFrame - iClosest))
                {
                    iClosest = lKey;
                }
            }

            return iClosest;
        }

        public bool WriteNewFile()
        {
            if (m_sNewChapters == "")
                return true;

            BSIState state = BSIState.Initial;
            TextReader reader = null;
            TextWriter writer = null;

            string m_sNewFile = m_sBSIFile + ".BSIWrapper";

            try
            {
                reader = new StreamReader(m_sBSIFile, System.Text.Encoding.Default);
                writer = new StreamWriter(m_sNewFile, false, System.Text.Encoding.Default);
            }
            catch
            {
                return false;
            }

            bool bFinished = false;

            while (reader.Peek() != -1)
            {
                string sLine = reader.ReadLine();

                state = CheckState(sLine, state);

                switch(state)
                {
                    case BSIState.Initial:
                    case BSIState.Main:
                    case BSIState.Movie:
                    case BSIState.Subtitles:
                        writer.WriteLine(sLine);
                        break;
                    case BSIState.Chapters:
                        writer.WriteLine(m_sNewChapters.TrimEnd(new char[] {'\r', '\n'}) + "\r\n");
                        bFinished = true;
                        break;
                }

                if (bFinished)
                    break;
            }

            reader.Close();
            writer.Close();

            File.Delete(m_sBSIFile);
            File.Move(m_sNewFile, m_sBSIFile);

            return true;
        }

        public string AVIFile
        {
            get
            {
                string sPath;

                sPath = Global.EnsureSlash(m_sDirectory) + m_sAVIFile;
                if (File.Exists(sPath))
                    return sPath;

                sPath = Global.EnsureSlash(Path.GetDirectoryName(m_sBSIFile)) + m_sAVIFile;
                if (File.Exists(sPath))
                    return sPath;

                return m_sAVIFile;
            }
        }

        enum BSIState
        {
            Initial,
            Main,
            Movie,
            Subtitles,
            Chapters
        }
	}
}
